package wahlchaos;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.text.NumberFormatter;

import wahlchaos.stoerungen.StimmenStoerung;
import wahlchaos.verteilungen.VerteilungsVerfahren;

public class Bundestagswahl {
	protected LinkedList<Partei> parteiListe;
	private final static int anzahlBundeslaender = 16;
	//private final static int mandate = 598;
	//private final static int mandate = 656;
	private int mandate;
	
	private int[][] erststimmenOrginal;
	private int[][] zweitstimmenOrginal;
	private String[] wahlkreise;
	private String[] parteien;
	private int[] bundeslaender;
	
	private int[][] zweitstimmen;
	private int[][] direktmandate;
	
	public final static int SPD = 0;
	public final static int CDU = 1;
	public final static int CSU = 2;
	public final static int GRUENE = 3;
	public final static int FDP = 4;
	public final static int PDS = 5;
	
	public final static String[] bundeslaenderNamen = {"SH","MP","HH","NS","HB","BRB","SA","B","NRW","SC","HE","TH","RP","BY","BW","SR"};
	
	public Bundestagswahl(int mandate) {
		this.mandate = mandate;		
	}
	
	public int getWahlkreisZahl() { return mandate/2; }
	public int getParteienZahl() { return parteien.length; }
	
	public int[] getBundeslaender() { return bundeslaender; }
	public String[] getWahlkreise() { return wahlkreise; }
	
	public int[][] getErststimmenOrginal() { return erststimmenOrginal; }
	public int[][] getZweitstimmenOrginal() { return zweitstimmenOrginal; }
	
	public void importData(String filename) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(filename));
		String line = null;
		
		// lese erste Zeile ein
		line=br.readLine();
		String[] parts = line.split(",");

		int anzahlParteien = (parts.length-1)/2;
		
		// ignoriere zweite Zeile
		line=br.readLine();

		parteien = new String[anzahlParteien];
		wahlkreise = new String[getWahlkreisZahl()];
		erststimmenOrginal = new int[getWahlkreisZahl()][anzahlParteien];
		zweitstimmenOrginal = new int[getWahlkreisZahl()][anzahlParteien];
		bundeslaender = new int[anzahlBundeslaender];
		
		int bundesland = 0;
		// Zaehler fuer Wahlkreise
		int wcounter = 0;
		while ((line=br.readLine())!=null) {
			if (line.startsWith(",,,")) {
				bundeslaender[bundesland++]=wcounter;
				continue;
			}
			String[] dummyParts = line.split(",");
			wahlkreise[wcounter] = dummyParts[1];
			
			for (int j=0;j<anzahlParteien;j++) {
				if (2*j+3 < dummyParts.length && !dummyParts[2*j+3].equalsIgnoreCase("")) {
					zweitstimmenOrginal[wcounter][j]=Integer.parseInt(dummyParts[2*j+3]);
				}	
				
				if (2*j+2 < dummyParts.length && !dummyParts[2*j+2].equalsIgnoreCase("")) {
					erststimmenOrginal[wcounter][j]=Integer.parseInt(dummyParts[2*j+2]);
				}
			}
			
			wcounter++;
		}
		
		// lese parteiennamen
		for (int p=0;p<anzahlParteien;p++) {
			parteien[p] = parts[2*p+2];
		}
	}
	
	public void mergeData(StimmenStoerung stoer) {
		direktmandate = new int[bundeslaender.length][erststimmenOrginal[0].length];
		zweitstimmen = new int[bundeslaender.length][zweitstimmenOrginal[0].length];
		
		// hole stoerungsmatrizen
		int[][] erststimmenStoer = stoer.getErststimmenStoerung();
		int[][] zweitstimmenStoer = stoer.getZweitstimmenStoerung();
		
		int wcounter = 0;
		for (int b=0; b<bundeslaender.length; b++) {
			for (;wcounter<bundeslaender[b];wcounter++) {
				// fuer direktmandate
				int maxJ = 0, maxErststimmen = 0;
				for (int j=0; j<erststimmenOrginal[wcounter].length; j++) {
					// bestimme maximum der erststimmen
					if (maxErststimmen < erststimmenOrginal[wcounter][j] + erststimmenStoer[wcounter][j]) {
						maxErststimmen = erststimmenOrginal[wcounter][j] + erststimmenStoer[wcounter][j];
						maxJ = j;
					}
					zweitstimmen[b][j] += zweitstimmenOrginal[wcounter][j] + zweitstimmenStoer[wcounter][j];
				}
				direktmandate[b][maxJ]++;
			}
		}
		
		this.parteiListe = new LinkedList<Partei>();
		for (int p=0;p<parteien.length;p++) {
			int[] erst = new int[anzahlBundeslaender];
			int[] zweit = new int[anzahlBundeslaender];
			for(int b=0;b<anzahlBundeslaender;b++) {
				erst[b] = direktmandate[b][p];
				zweit[b] = zweitstimmen[b][p];
			}
			
			parteiListe.add(new Partei(parteien[p],erst.clone(),zweit.clone()));
		}
	}
			
	public void bestimmeSitzverteilung(VerteilungsVerfahren v) {
		int stimmenTotal = 0;
		for (Partei p : parteiListe) stimmenTotal += p.getZweitstimmenZahl();
		for (Partei p : parteiListe) p.berechneProzente(stimmenTotal);
		
		int freieMandate = mandate;
		LinkedList<Partei> fraktionsListe = new LinkedList<Partei>(); 
		LinkedList<Partei> nichtFraktionsListe = new LinkedList<Partei>();
		for (Partei p : parteiListe) {
			if (!p.istFraktion()) {
				freieMandate -= p.getErststimmenZahl();
				p.setzeMandate(p.getErststimmenZahl());
				nichtFraktionsListe.add(p);
			} else {
				fraktionsListe.add(p);
			}
		}
		
		int[] stimmenTemp = new int[fraktionsListe.size()];
		for (int i=0;i<stimmenTemp.length;i++) {
			stimmenTemp[i] = parteiListe.get(i).getZweitstimmenZahl();
		}
		v.setzeDaten(stimmenTemp, freieMandate);
		// oberverteilung
		int[] sitze = v.berechneSitze();
		// unterverteilung
		for (int i=0;i<sitze.length;i++) {
			parteiListe.get(i).setzeMandate(sitze[i]);
			parteiListe.get(i).berechneLandesverteilung(v);
		}
	}
	
	public LinkedList<Partei> getParteiListe() {
		return this.parteiListe;
	}
	
	public int dist(Bundestagswahl b) {
		int dist = 0;
		boolean found = false;
		for (Partei p1 : this.parteiListe) {
			if (p1.hatSitze()) {
				found = false;
				for (Partei p2 : b.parteiListe) {
					if (p1.name.equals(p2.name)) {
						dist += Math.abs(p1.mandate-p2.mandate);
						found = true;
						break;
					}
				}
				if (!found) dist+=p1.mandate;
			} else {
				for (Partei p2 : b.parteiListe) {
					if (p1.name.equals(p2.name)) {
						dist += p2.mandate;
						break;
					}
				}
			}
		}
		
		return dist;
	}
	
	public String diff(Bundestagswahl b) {
		StringBuffer diff = new StringBuffer();
		boolean found = false;
		for (Partei p1 : this.parteiListe) {
			found = false;
			if (p1.hatSitze()) {
				for (Partei p2 : b.parteiListe) {
					if (p1.name.equals(p2.name)) {
						diff.append(p1.name + " : " + (-p1.mandate+p2.mandate));
						found = true;
					}
				}
				if (!found) diff.append(p1.name + " : " + (-p1.mandate));
				diff.append (" | ");
			}
		}
		
		return diff.toString();
	}
	
	public Vector<Integer> diff2(Bundestagswahl b) {
		Vector<Integer> diff = new Vector<Integer>(6);
		boolean found = false;
		for (Partei p1 : this.parteiListe) {
			found = false;
			if (p1.hatSitze()) {
				for (Partei p2 : b.parteiListe) {
					if (p1.name.equals(p2.name)) {
						diff.add(-p1.mandate+p2.mandate);
						found = true;
					}
				}
				if (!found) diff.add(-p1.mandate);
			}
		}
		
		return diff;
	}
	
	public static String getFormattedNumber(double a) {
		NumberFormatter nf = new NumberFormatter(new DecimalFormat("0.00"));
		try {
			return nf.valueToString(a);
		} catch(ParseException e) {
			e.printStackTrace();
		}
		return "";
	}
}
