package wahlchaos;

import wahlchaos.verteilungen.VerteilungsVerfahren;

public class Partei {
	protected String name;
	protected int[] erststimmen;
	private int summeErststimmen;
	protected int[] zweitstimmen;
	private int summeZweitstimmen;
	private double prozentZweitstimmen;
	protected boolean minoritaet;
	
	protected int mandate;
	protected int[] mandatsVerteilung;
	protected int ueberhangMandate;
	protected int[] ueberhangVerteilung;
	
	public Partei(String _name, int[] _erststimmen, int[] _zweitstimmen) {
		this(_name,_erststimmen,_zweitstimmen,false);
	}
	
	public Partei(String _name, int[] _erststimmen, int[] _zweitstimmen, boolean _minoritaet) {
		this.name = _name;
		this.erststimmen = _erststimmen;
		this.zweitstimmen = _zweitstimmen;
		this.minoritaet = _minoritaet;
		
		this.summeErststimmen = 0;
		for (int s:erststimmen) summeErststimmen += s;
		
		this.summeZweitstimmen = 0;
		for (int s:zweitstimmen) summeZweitstimmen += s;
	}
	
	public int[] berechneLandesverteilung(VerteilungsVerfahren v) {
		v.setzeDaten(this.zweitstimmen, this.mandate);
		
		int[] mandatsVerteilung = v.berechneSitze();
		this.ueberhangVerteilung = new int[mandatsVerteilung.length];
		for (int i=0;i<mandatsVerteilung.length; i++) {
			/*if (mandatsVerteilung[i]<this.erststimmen[i]) {
				System.out.println((this.erststimmen[i]-mandatsVerteilung[i])+" Ueberhangmandate fuer " + this.name + " in " + i);
			}*/
			ueberhangMandate += Math.max(this.erststimmen[i]-mandatsVerteilung[i],0);
			this.ueberhangVerteilung[i] = Math.max(this.erststimmen[i]-mandatsVerteilung[i],0);
			mandatsVerteilung[i] = Math.max(mandatsVerteilung[i],this.erststimmen[i]);
		}
		//System.out.println("--------");
		this.mandate = 0;
		for (int s:mandatsVerteilung) mandate += s;
		this.mandatsVerteilung = mandatsVerteilung;
		
		return null;
	}
	
	public boolean istFraktion() {
		return this.minoritaet || this.summeErststimmen >= 3 || this.prozentZweitstimmen >= 0.05; 
	}
	
	public int getErststimmenZahl() {
		return this.summeErststimmen;
	}
	
	public int getZweitstimmenZahl() {
		return this.summeZweitstimmen;
	}

	public void berechneProzente(int stimmenTotal) {
		prozentZweitstimmen = (double) this.getZweitstimmenZahl() / stimmenTotal;
	}

	public void setzeMandate(int m) {
		this.mandate = m;
	}
	
	public String toString() {
		String ret = name + " mit " + mandate + " Mandate: ";
		/*if (mandatsVerteilung!=null) {
			for (int v:mandatsVerteilung) ret += v+ ",";
		}*/
		return ret;
	}
	
	public String getUeberhangVerteilung(boolean verbose) {
		StringBuffer sb = new StringBuffer();
		if (this.ueberhangVerteilung!=null) {
			int b=0;
			for (int u : this.ueberhangVerteilung) {
				if(verbose) {
					sb.append(Bundestagswahl.bundeslaenderNamen[b++]);
					sb.append(":");
				}
				sb.append(u);
				sb.append(",");
			}
			return sb.substring(0,sb.length()-1);
		} else {
			return "";
		}
		
	}

	public boolean hatSitze() {
		//return getErststimmenZahl() > 0;
		return this.mandate > 0;
	}
}
