package wahlchaos;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.StringTokenizer;

import javax.swing.text.NumberFormatter;

import wahlchaos.stoerungen.*;
import wahlchaos.verteilungen.DHondt;
import wahlchaos.verteilungen.HareNiemeyer;
import wahlchaos.verteilungen.SainteLague;
import wahlchaos.verteilungen.VerteilungsVerfahren;

public class Sim {
	
	private static void usage() {
		System.err.println("Parameter:");
		System.err.println("(verteilung,datei,stoerung)+");
	}
	
	private static void output(LinkedList<Partei> p0, boolean verbose) {
		int s = 0;
		for (Partei p : p0) s+=p.mandate;
		System.out.println("\nInsgesamt " + s + " Sitze\n");
		for (Partei p : p0) {
			if (p.hatSitze()) {
				String prozent = Bundestagswahl.getFormattedNumber(p.mandate/(double)s*100);
				//System.out.println(p + " ["+prozent+" %], "+p.ueberhangMandate + "UeM ("+p.getUeberhangVerteilung()+")");
				
				
				// reserve 30 chars for first column
				StringBuffer sb = new StringBuffer(p.toString());
				while(sb.length()<30) sb.append(" ");
				System.out.println( sb.toString() +prozent+" %\t"+p.ueberhangMandate + "UeM ("+p.getUeberhangVerteilung(verbose)+")");
			}
		}
		System.out.println();
	}
	
	private static void outputHeader(VerteilungsVerfahren v, String datei, StimmenStoerung s) {
		System.out.println("------------------------------------------------------------------------");
		System.out.println(v.getName() + " | " + datei + " | " + s.getName());
		if (!s.getInformation().equals("")) {
			System.out.println(s.getInformation());
		}
		System.out.println("------------------------------------------------------------------------");
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		int anzahl = args.length;
		
		if (anzahl<1) {
			usage();
			System.exit(0);
		}
		
		int[] mandate = new int[anzahl];
		String[] verteilungsVerfahren = new String[anzahl];
		String[] dateiNamen = new String[anzahl];
		String[] stoerungen = new String[anzahl];
		boolean verbose = args[anzahl-1].equals("v");
		if (verbose) anzahl--;
		
		final String delim = ","; 
		StringTokenizer st = null;
		for (int i=0;i<anzahl;i++) {
			st = new StringTokenizer(args[i],delim);
			mandate[i] = Integer.parseInt(st.nextToken());
			verteilungsVerfahren[i] = st.nextToken();
			dateiNamen[i] = st.nextToken();
			stoerungen[i] = st.nextToken();
		}
		
		Bundestagswahl[] bw = new Bundestagswahl[anzahl];
		for (int i=0;i<anzahl;i++) {
			bw[i] = new Bundestagswahl(mandate[i]);
			
			// lese daten aus CSV ein
			try {
				bw[i].importData(dateiNamen[i]);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		VerteilungsVerfahren[] v = new VerteilungsVerfahren[anzahl];
		StimmenStoerung[] s = new StimmenStoerung[anzahl];
		LinkedList<Partei>[] parteiListen = (LinkedList<Partei>[]) new LinkedList[anzahl];
		
		// bestimmte verteilungsverfahren
		for (int i=0;i<anzahl;i++) {
			if (verteilungsVerfahren[i].equalsIgnoreCase("hare")) {
				v[i] = new HareNiemeyer();
			} else if (verteilungsVerfahren[i].equalsIgnoreCase("sainte")) {
				v[i] = new SainteLague();
			} else if (verteilungsVerfahren[i].equalsIgnoreCase("dhondt")) {
				v[i] = new DHondt();
			}
		}
		
		// bestimme stimmenstoerung
		for (int i=0;i<anzahl;i++) {
			if (stoerungen[i].equalsIgnoreCase("dresden2005")) {
				s[i] = new StoerungDresden2005(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl());
			} else if (stoerungen[i].equalsIgnoreCase("ost")) {
				s[i] = new StoerungOst(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].equalsIgnoreCase("west")) {
				s[i] = new StoerungWest(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].equalsIgnoreCase("gesamt")) {
				s[i] = new StoerungGesamt(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].equalsIgnoreCase("zweitstimmen")) {
				s[i] = new StoerungZweitstimmen(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].equalsIgnoreCase("erststimmen")) {
				s[i] = new StoerungErststimmen(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].equalsIgnoreCase("beteiligung")) {
				s[i] = new StoerungWahlbeteiligung(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal());
			} else if (stoerungen[i].startsWith("zk")) {
				s[i] = new StoerungZufallKreis(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal(),Integer.parseInt(stoerungen[i].substring(2)));
			} else if (stoerungen[i].startsWith("xml")) {
				s[i] = new XMLStoerung(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl(),bw[i].getErststimmenOrginal(),bw[i].getZweitstimmenOrginal(), stoerungen[i].substring(4,stoerungen[i].length()-1));
			} else {
				s[i] = new StoerungNull(bw[i].getWahlkreisZahl(),bw[i].getParteienZahl());
			} 
		}
		
		for (int i=0;i<anzahl;i++) {
			bw[i].mergeData(s[i]);

			bw[i].bestimmeSitzverteilung(v[i]);
			parteiListen[i] = bw[i].getParteiListe();
			outputHeader(v[i],dateiNamen[i],s[i]);
			output(parteiListen[i],verbose);
		}
	}

}
