package wahlchaos.stoerungen;

import java.util.Random;

import wahlchaos.Bundestagswahl;

public abstract class StimmenStoerung {
	protected int[][] erststimmenStoer;
	protected int[][] zweitstimmenStoer;
	protected int[][] erststimmenOrginal;
	protected int[][] zweitstimmenOrginal;
	
	protected String name;
	protected String info = "";
	
	public String getName() {
		return this.name;
	}
	
	public String getInformation() {
		return this.info;
	}
	
	public StimmenStoerung(int wahlkreise, int parteien, int[][] erst, int[][] zweit) {
		erststimmenStoer = new int[wahlkreise][parteien];
		zweitstimmenStoer = new int[wahlkreise][parteien];
		erststimmenOrginal = erst;
		zweitstimmenOrginal = zweit;
	}
	
	public int[][] getErststimmenStoerung() {
		return erststimmenStoer;
	}
	
	public int[][] getZweitstimmenStoerung() {
		return zweitstimmenStoer;
	}
	
	protected void stoereKreis(int kreis, double prozent) {
		for (int p=0; p<erststimmenStoer[kreis].length; p++) {
			int arith = erststimmenOrginal[kreis][p] + zweitstimmenOrginal[kreis][p];
			arith>>=1;
			erststimmenStoer[kreis][p] = (int)(prozent * arith);
			zweitstimmenStoer[kreis][p] = (int)(prozent * arith);
		}
		
		Random r = new Random();
		
		for (int p=0; p<erststimmenStoer[kreis].length; p++) {
			erststimmenStoer[kreis][p] = (int)(r.nextGaussian() * erststimmenStoer[kreis][p]);
			zweitstimmenStoer[kreis][p] = (int)(r.nextGaussian() * zweitstimmenStoer[kreis][p]);
		}
		
		info = "Erststimmenfluktuation: " + Bundestagswahl.getFormattedNumber((double)foldAddAbs(erststimmenStoer[kreis]) / foldAddAbs(erststimmenOrginal[kreis]) * 100);
		info += " | Zweistimmenfluktuation: " + Bundestagswahl.getFormattedNumber((double)foldAddAbs(zweitstimmenStoer[kreis]) / foldAddAbs(zweitstimmenOrginal[kreis]) * 100);
	}
	
	
	protected void erhoeheKreis(int kreis, int partei, double prozent) {
		erststimmenStoer[kreis][partei] = (int)(prozent * erststimmenOrginal[kreis][partei]);
		zweitstimmenStoer[kreis][partei] = (int)(prozent * zweitstimmenOrginal[kreis][partei]);
	}
	
	protected static int foldAddAbs(int[] array) {
		int sum=0;
		for (int a : array) sum+=Math.abs(a);
		return sum;
	}

	
}
